package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminPendaftaranHandler struct {
	pendaftaranUsecase usecase.PendaftaranUsecase
}

func NewAdminPendaftaranHandler(cfg config.PendaftaranTransportConfig) *AdminPendaftaranHandler {
	pendaftaranRepository := repository.NewPendaftaranRepository(cfg.DBWrite, cfg.DBRead)
	pendaftaranUsecase := usecase.NewPendaftaranUseCase(pendaftaranRepository)

	return &AdminPendaftaranHandler{
		pendaftaranUsecase: pendaftaranUsecase,
	}

}

func (t *AdminPendaftaranHandler) MountAdmin(group *echo.Group) {
	group.POST("/pendaftaran", t.AddPendaftaran)
	group.GET("/pendaftaran", t.GetPendaftaran)
	group.GET("/pendaftaran-by-dosen", t.GetPendaftaranByIdDosen)
	group.DELETE("/pendaftaran-delete", t.DeletePendaftaran)
	group.POST("/pendaftaran-update-status", t.UpdateStatusPendaftaran)
	group.POST("/pendaftaran-update-interview", t.UpdateWaktuInterview)
	group.GET("/pendaftaran-by-periode", t.GetPendaftaranByPeriode)
	group.GET("/pendaftaran-by-id", t.GetPendaftaranById)
}

// AddPendaftaran adds a new pendaftaran record.
// @Summary Add pendaftaran
// @Description Adds a new pendaftaran record.
// @Tags Pendaftaran
// @Accept json
// @Produce json
// @Param request body entity.PendaftaranAddRequest true "Pendaftaran Add Request"
// @Success 201 {object} entity.Pendaftaran
// @Router /pendaftaran [post]
func (t *AdminPendaftaranHandler) AddPendaftaran(c echo.Context) error {
	request := new(entity.PendaftaranAddRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	target := &models.PendaftaranTA{
		IdTopik:     request.IdTopik,
		IdMahasiswa: request.IdMahasiswa,
		IdDosen:     request.IdDosen,
	}

	newPendaftaran, err := t.pendaftaranUsecase.AddPendaftaran(*target)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Pendaftaran added successfully", newPendaftaran))
}

// GetPendaftaranByIdDosen retrieves a list of pendaftaran records by dosen ID.
// @Summary Get pendaftaran by dosen ID
// @Description Retrieves a list of pendaftaran records by dosen ID.
// @Tags Pendaftaran
// @Accept json
// @Produce json
// @Param id query string true "ID of the dosen"
// @Success 200 {object} []entity.Pendaftaran
// @Router /pendaftaran-get-by-dosen [get]
func (t *AdminPendaftaranHandler) GetPendaftaran(c echo.Context) error {
	listGroupLimit, err := t.pendaftaranUsecase.GetPendaftaran()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", listGroupLimit))
}

func (t *AdminPendaftaranHandler) GetPendaftaranByIdDosen(c echo.Context) error {
	id := c.QueryParam("id")
	listGroupLimit, err := t.pendaftaranUsecase.GetPendaftaranByIdDosen(id)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", listGroupLimit))
}

func (t *AdminPendaftaranHandler) GetPendaftaranById(c echo.Context) error {
	id := c.QueryParam("id")
	listGroupLimit, err := t.pendaftaranUsecase.GetPendaftaranById(id)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", listGroupLimit))
}

// DeletePendaftaran deletes a pendaftaran record.
// @Summary Delete pendaftaran
// @Description Deletes a pendaftaran record.
// @Tags Pendaftaran
// @Accept json
// @Produce json
// @Param id query string true "ID of the pendaftaran"
// @Success 200 {object} interface{}
// @Router /pendaftaran-delete [get]
func (t *AdminPendaftaranHandler) DeletePendaftaran(c echo.Context) error {
	id := c.QueryParam("id")
	listGroupLimit, err := t.pendaftaranUsecase.DeletePendaftaran(id)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully deleted", listGroupLimit))
}

// UpdateStatusPendaftaran updates the status of a pendaftaran record.
// @Summary Update pendaftaran status
// @Description Updates the status of a pendaftaran record.
// @Tags Pendaftaran
// @Accept json
// @Produce json
// @Param request body entity.UpdatePendaftaranRequest true "Update Pendaftaran Status Request"
// @Success 201 {object} entity.Pendaftaran
// @Router /pendaftaran-update-status [post]
func (t *AdminPendaftaranHandler) UpdateStatusPendaftaran(c echo.Context) error {
	request := new(entity.UpdatePendaftaranRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	newPendaftaran, err := t.pendaftaranUsecase.UpdateStatusPendaftaran(request.Id, request.Status)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Pendaftaran status updated successfully", newPendaftaran))
}

// UpdateWaktuInterview updates the interview time of a pendaftaran record.
// @Summary Update pendaftaran interview time
// @Description Updates the interview time of a pendaftaran record.
// @Tags Pendaftaran
// @Accept json
// @Produce json
// @Param request body entity.WaktuInterviewUpdateRequest true "Update Pendaftaran Interview Time Request"
// @Success 201 {object} entity.Pendaftaran
// @Router /pendaftaran-update-interview [post]
func (t *AdminPendaftaranHandler) UpdateWaktuInterview(c echo.Context) error {
	request := new(entity.WaktuInterviewUpdateRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	newPendaftaran, err := t.pendaftaranUsecase.UpdateWaktuInterview(request.Id, request.InterviewAt)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Pendaftaran interview time updated successfully", newPendaftaran))
}

func (t *AdminPendaftaranHandler) GetPendaftaranByPeriode(c echo.Context) error {
	idPeriode := c.QueryParam("id_periode")
	pendaftaran, err := t.pendaftaranUsecase.GetPendaftaranByPeriode(idPeriode)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", pendaftaran))
}
