package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/period_pendaftaran/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/period_pendaftaran/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/period_pendaftaran/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/period_pendaftaran/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminPeriodPendaftaranHandler struct {
	periodPendaftaranUsecase usecase.PeriodPendaftaranUsecase
}

func NewAdminPeriodPendaftaranHandler(cfg config.PeriodPendaftaranConfig) *AdminPeriodPendaftaranHandler {
	periodPendaftaranRepository := repository.NewPeriodRepo(cfg.DBWrite, cfg.DBRead)
	periodPendaftaranUsecase := usecase.NewPeriodUseCase(periodPendaftaranRepository)

	return &AdminPeriodPendaftaranHandler{
		periodPendaftaranUsecase: periodPendaftaranUsecase,
	}

}

func (t *AdminPeriodPendaftaranHandler) MountAdmin(group *echo.Group) {
	group.POST("/periode-pendaftaran", t.AddPeriod)
	group.GET("/periode-pendaftaran", t.GetPeriod)
	group.DELETE("/periode-pendaftaran", t.DeletePeriod)
	group.GET("/periode-pendaftaran-current", t.GetPeriodCurrent)
	group.PATCH("/periode-pendaftaran-live", t.SetLive)
	group.GET("/periode-pendaftaran-live", t.GetLive)
}

// AddPeriod adds a new period for pendaftaran.
// @Summary Add period for pendaftaran
// @Description Adds a new period for pendaftaran.
// @Tags Periode Pendaftaran
// @Accept json
// @Produce json
// @Param batch body entity.PeriodPendaftaran true "Period Pendaftaran object"
// @Success 200 {object} entity.PeriodPendaftaran "Period pendaftaran added successfully"
// @Router /periode-pendaftaran [post]
func (t *AdminPeriodPendaftaranHandler) AddPeriod(c echo.Context) error {
	request := new(entity.PeriodPendaftaran)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	target := &entity.PeriodPendaftaran{
		StartDate: request.StartDate,
		EndDate:   request.EndDate,
	}
	newBatch, err := t.periodPendaftaranUsecase.AddPeriod(*target)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Period pendaftaran added successfully", newBatch))
}

// GetPeriod retrieves a list of period for pendaftaran.
// @Summary Get period for pendaftaran
// @Description Retrieves a list of bperiod for pendaftaran.
// @Tags Periode Pendaftaran
// @Accept json
// @Produce json
// @Success 200 {object} []entity.PeriodPendaftaran "Period pendaftaran successfully retreived"
// @Router /periode-pendaftaran [get]
func (t *AdminPeriodPendaftaranHandler) GetPeriod(c echo.Context) error {
	period, err := t.periodPendaftaranUsecase.GetPeriod()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Period pendaftaran successfully retreived", period))
}

// DeletePeriod deletes a period for pendaftaran.
// @Summary Delete period for pendaftaran
// @Description Deletes a period for pendaftaran.
// @Tags Periode Pendaftaran
// @Accept json
// @Produce json
// @Param batch body entity.PeriodPendaftaran true "Period Pendaftaran object"
// @Success 200 {object} entity.PeriodPendaftaran "Period pendaftaran successfully deleted"
// @Router /periode-pendaftaran [delete]
func (t *AdminPeriodPendaftaranHandler) DeletePeriod(c echo.Context) error {
	request := new(entity.PeriodPendaftaran)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	period, err := t.periodPendaftaranUsecase.DeletePeriod(request.IDPeriod)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Period pendaftaran successfully deleted", period))
}

// GetPeriodCurrent retrieves a list of period for pendaftaran that current time is between its startDate and endDate.
// @Summary Get periode pendaftaran that current time is between its startDate and endDate.
// @Description Retrieves a list of period for pendaftaran.
// @Tags Periode Pendaftaran
// @Accept json
// @Produce json
// @Success 200 {object} entity.PeriodPendaftaran "Period pendaftaran successfully retreived"
// @Router /periode-pendaftaran-current [get]
func (t *AdminPeriodPendaftaranHandler) GetPeriodCurrent(c echo.Context) error {
	period, err := t.periodPendaftaranUsecase.GetPeriodCurrent()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Period pendaftaran successfully retreived", period))
}

// SetLive sets a period for pendaftaran to be live.
// @Summary Set Period to live
// @Description Set Period to live.
// @Tags Periode Pendaftaran
// @Accept json
// @Produce json
// @Param batch body entity.PeriodPendaftaran true "Period Pendaftaran object"
// @Success 200 {object} entity.PeriodPendaftaran "Periode pendaftaran successfully updated to live"
// @Router /periode-pendaftaran-live [patch]
func (t *AdminPeriodPendaftaranHandler) SetLive(c echo.Context) error {
	request := new(entity.PeriodPendaftaran)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	batch, err := t.periodPendaftaranUsecase.SetLivePeriode(request.IDPeriod)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Periode pendaftaran successfully updated to live", batch))
}

// GetLive retrieves a period for pendaftaran that its status is live.
// @Summary Get periode pendaftaran that is live.
// @Description Retrieves a period for pendaftaran.
// @Tags Periode Pendaftaran
// @Accept json
// @Produce json
// @Success 200 {object} []entity.PeriodPendaftaran "Periode pendaftaran successfully retreived"
// @Router /periode-pendaftaran-live [get]
func (t *AdminPeriodPendaftaranHandler) GetLive(c echo.Context) error {
	batch, err := t.periodPendaftaranUsecase.GetPeriodeLive()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Periode pendaftaran successfully retreived", batch))
}
