package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

// AdminTopicAllocationHandler handles HTTP requests related to topic allocation management by admin.
type AdminTopicAllocationHandler struct {
	topicUsecase usecase.TopicUsecase
}

// NewAdminTopicAllocationHandler creates a new instance of AdminTopicAllocationHandler.
func NewAdminTopicAllocationHandler(cfg config.TopicAllocationTransportConfig) *AdminTopicAllocationHandler {
	topicAllocationRepository := repository.NewTopicAllocationRepository(cfg.DBWrite, cfg.DBRead)
	topicUseCase := usecase.NewTopicUc(topicAllocationRepository)

	return &AdminTopicAllocationHandler{
		topicUsecase: topicUseCase,
	}
}

// MountAdmin mounts the admin routes for topic allocation management.
func (admin *AdminTopicAllocationHandler) MountAdmin(group *echo.Group) {
	group.GET("/alokasi-topik/:id", admin.GetTopicByID)
	group.GET("/alokasi-topik", admin.GetTopics)
	group.POST("/alokasi-topik", admin.AddTopic)
	group.PUT("/alokasi-topik/:id", admin.UpdateTopic)
	group.DELETE("/alokasi-topik/:id", admin.DeleteTopic)
}

// GetTopics retrieves a list of topics.
// @Summary Retrieve topics
// @Description Retrieves a list of topics.
// @Tags Topics
// @Produce json
// @Param page query int false "Page number"
// @Param limit query int false "Items per page"
// @Param search query string false "Search query"
// @Param id query string false "Topic ID"
// @Success 200 {object} []entity.Topik
// @Router /alokasi-topik [get]
func (admin *AdminTopicAllocationHandler) GetTopics(c echo.Context) error {
	param, err := utils.GetPageLimitSearchID(c.QueryParams())

	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	list, err := admin.topicUsecase.GetTopic(param)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topics retrieved", list))
}

// GetTopicByID retrieves a topic by ID.
// @Summary Retrieve a topic by ID
// @Description Retrieves a topic by its ID.
// @Tags Topics
// @Produce json
// @Param id path string true "Topic ID"
// @Success 200 {object} entity.Topik
// @Router /alokasi-topik/{id} [get]
func (admin *AdminTopicAllocationHandler) GetTopicByID(c echo.Context) error {
	id := c.Param("id")

	if id != "" {
		item, err := admin.topicUsecase.GetTopicByID(id)

		if err != nil {
			return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
		}

		return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topic retrieved", item))
	} else {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, "Incorrect URL", nil))
	}
}

// AddTopic adds a new topic.
// @Summary Add all new topics
// @Description Adds all new topics.
// @Tags Topics
// @Accept json
// @Produce json
// @Param topics body []entity.Topik true "Topic object to add"
// @Success 200 {object} []entity.TopikPostPutReqData
// @Router /alokasi-topik [post]
func (admin *AdminTopicAllocationHandler) AddTopic(c echo.Context) error {
	var topics []entity.TopikPostPutReqData

	if err := c.Bind(&topics); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	topics, err := admin.topicUsecase.AddTopic(topics)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Unable to add topics", nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topic added successfully", topics))
}

// UpdateTopic updates an existing topic.
// @Summary Update an existing topic
// @Description Updates an existing topic.
// @Tags Topics
// @Accept json
// @Produce json
// @Param id path string true "Topic ID"
// @Param topic body entity.Topik true "Updated topic object"
// @Success 200 {object} entity.Topik
// @Router /alokasi-topik/{id} [put]
func (admin *AdminTopicAllocationHandler) UpdateTopic(c echo.Context) error {
	id := c.Param("id")

	if id != "" {
		topic := new(entity.TopikPostPutReqData)
		c.Bind(topic)

		newItem, err := admin.topicUsecase.UpdateTopic(*topic, id)

		if err != nil {
			return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Failed to update topic", nil))
		}

		return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topic updated", newItem))
	} else {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, "Please provide topic id", nil))
	}
}

// DeleteTopic deletes a topic by ID.
// @Summary Delete a topic by ID
// @Description Deletes a topic by its ID.
// @Tags Topics
// @Produce json
// @Param id path string true "Topic ID"
// @Success 200 {object} nil
// @Router /alokasi-topik/{id} [delete]
func (admin *AdminTopicAllocationHandler) DeleteTopic(c echo.Context) error {
	id := c.Param("id")

	if id != "" {
		err := admin.topicUsecase.DeleteTopic(id)

		if err != nil {
			return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Failed to delete topic", nil))
		}

		return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topic deleted successfully", nil))
	} else {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, "Please provide topic id", nil))
	}
}
